@@ -24,13 +24,13 @@
  *
  * ------------------------------------------------------------
  * Main author:
- * - Daniel Stenberg <Daniel.Stenberg@haxx.nu>
+ * - Daniel Stenberg <daniel@haxx.se>
  *
- * 	http://curl.haxx.nu
+ * 	http://curl.haxx.se
  *
  * $Source: /cvsroot/curl/lib/escape.c,v $
- * $Revision: 1.3 $
- * $Date: 2000/03/20 10:22:12 $
+ * $Revision: 1.6 $
+ * $Date: 2000/08/31 12:03:04 $
  * $Author: bagder $
  * $State: Exp $
  * $Locker:  $
@@ -41,6 +41,9 @@
 /* Escape and unescape URL encoding in strings. The functions return a new
  * allocated string or NULL if an error occurred.  */
 
+#include "setup.h"
+#include <curl/curl.h>
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -81,25 +84,31 @@ char *curl_escape(char *string)
    return ns;
 }
 
-char *curl_unescape(char *string)
+char *curl_unescape(char *string, int length)
 {
-   int alloc = strlen(string)+1;
+   int alloc = (length?length:strlen(string))+1;
    char *ns = malloc(alloc);
    unsigned char in;
    int index=0;
    int hex;
-   
-
-   while(*string) {
+   char querypart=FALSE; /* everything to the right of a '?' letter is
+                            the "query part" where '+' should become ' '.
+                            RFC 2316, section 3.10 */
+  
+   while(--alloc) {
       in = *string;
-      if('+' == in)
-	 in = ' ';
+      if(querypart && ('+' == in))
+         in = ' ';
+      else if(!querypart && ('?' == in)) {
+        /* we have "walked in" to the query part */
+        querypart=TRUE;
+      }
       else if('%' == in) {
-	 /* encoded part */
-	 if(sscanf(string+1, "%02X", &hex)) {
-	    in = hex;
-	    string+=2;
-	 }
+        /* encoded part */
+        if(sscanf(string+1, "%02X", &hex)) {
+          in = hex;
+          string+=2;
+        }
       }
 
       ns[index++] = in;
