@@ -29,8 +29,8 @@
  * 	http://curl.haxx.se
  *
  * $Source: /cvsroot/curl/lib/http.c,v $
- * $Revision: 1.23 $
- * $Date: 2000/08/24 14:26:33 $
+ * $Revision: 1.27 $
+ * $Date: 2000/09/21 08:48:48 $
  * $Author: bagder $
  * $State: Exp $
  * $Locker:  $
@@ -134,9 +134,58 @@ bool static checkheaders(struct UrlData *data, char *thisheader)
   return FALSE;
 }
 
+/*
+ * GetHTTPProxyTunnel() requires that we're connected to a HTTP proxy. This
+ * function will issue the necessary commands to get a seamless tunnel through
+ * this proxy. After that, the socket can be used just as a normal socket.
+ */
+
+CURLcode GetHTTPProxyTunnel(struct UrlData *data, int tunnelsocket,
+                            char *hostname, int remote_port)
+{
+  int httperror=0;
+  int subversion=0;
+
+  infof(data, "Establish HTTP proxy tunnel to %s:%d\n", hostname, remote_port);
+
+  /* OK, now send the connect statment */
+  sendf(tunnelsocket, data,
+        "CONNECT %s:%d HTTP/1.0\015\012"
+        "%s"
+        "%s"
+        "\r\n",
+        hostname, remote_port,
+        (data->bits.proxy_user_passwd)?data->ptr_proxyuserpwd:"",
+        (data->useragent?data->ptr_uagent:"")
+        );
+
+  /* wait for the proxy to send us a HTTP/1.0 200 OK header */
+  while(GetLine(tunnelsocket, data->buffer, data)) {
+    if('\r' == data->buffer[0])
+      break; /* end of headers */
+    if(2 == sscanf(data->buffer, "HTTP/1.%d %d",
+                   &subversion,
+                   &httperror)) {
+      ;
+    }
+  }
+  if(200 != httperror) {
+    if(407 == httperror)
+      /* Added Nov 6 1998 */
+      failf(data, "Proxy requires authorization!");
+    else 
+      failf(data, "Received error code %d from proxy", httperror);
+    return CURLE_READ_ERROR;
+  }
+
+  infof (data, "Proxy replied to CONNECT request\n");
+  return CURLE_OK;
+}
+
 CURLcode http_connect(struct connectdata *conn)
 {
   struct UrlData *data;
+  CURLcode result;
 
   data=conn->data;
 
@@ -146,59 +195,28 @@ CURLcode http_connect(struct connectdata *conn)
    * us to the host we want to talk to.  Only after the connect
    * has occured, can we start talking SSL
    */
-   if (conn->protocol & PROT_HTTPS) {
-     if (data->bits.httpproxy) {
+  if (conn->protocol & PROT_HTTPS) {
+    if (data->bits.httpproxy) {
+      /* HTTPS through a proxy can only be done with a tunnel */
+      result = GetHTTPProxyTunnel(data, data->firstsocket,
+                                  data->hostname, data->remote_port);
+      if(CURLE_OK != result)
+        return result;
+    }
 
-        /* OK, now send the connect statment */
-        sendf(data->firstsocket, data,
-              "CONNECT %s:%d HTTP/1.0\015\012"
-              "%s"
-	      "%s"
-              "\r\n",
-              data->hostname, data->remote_port,
-              (data->bits.proxy_user_passwd)?data->ptr_proxyuserpwd:"",
-	      (data->useragent?data->ptr_uagent:"")
-              );
-
-        /* wait for the proxy to send us a HTTP/1.0 200 OK header */
-	/* Daniel rewrote this part Nov 5 1998 to make it more obvious */
-	{
-	  int httperror=0;
-	  int subversion=0;
-	  while(GetLine(data->firstsocket, data->buffer, data)) {
-	    if('\r' == data->buffer[0])
-	      break; /* end of headers */
-	    if(2 == sscanf(data->buffer, "HTTP/1.%d %d",
-			   &subversion,
-			   &httperror)) {
-	      ;
-	    }
-	  }
-	  if(200 != httperror) {
-	    if(407 == httperror)
-	      /* Added Nov 6 1998 */
-	      failf(data, "Proxy requires authorization!");
-	    else 
-	      failf(data, "Received error code %d from proxy", httperror);
-	    return CURLE_READ_ERROR;
-	  }
-	}
-        infof (data, "Proxy has replied to CONNECT request\n");
-     }
-
-      /* now, perform the SSL initialization for this socket */
-     if(UrgSSLConnect (data)) {
-       return CURLE_SSL_CONNECT_ERROR;
-     }
+    /* now, perform the SSL initialization for this socket */
+    if(UrgSSLConnect (data)) {
+      return CURLE_SSL_CONNECT_ERROR;
+    }
   }
 
-   if(data->bits.user_passwd && !data->bits.this_is_a_follow) {
-     /* Authorization: is requested, this is not a followed location, get the
-        original host name */
-     data->auth_host = strdup(data->hostname);
-   }
+  if(data->bits.user_passwd && !data->bits.this_is_a_follow) {
+    /* Authorization: is requested, this is not a followed location, get the
+       original host name */
+    data->auth_host = strdup(data->hostname);
+  }
 
-   return CURLE_OK;
+  return CURLE_OK;
 }
 
 /* called from curl_close() when this struct is about to get wasted, free
@@ -269,7 +287,7 @@ CURLcode http(struct connectdata *conn)
   }
 
   if((data->bits.user_passwd) && !checkheaders(data, "Authorization:")) {
-    char authorization[512];
+    char *authorization;
 
     /* To prevent the user+password to get sent to other than the original
        host due to a location-follow, we do some weirdo checks here */
@@ -277,9 +295,12 @@ CURLcode http(struct connectdata *conn)
        !data->auth_host ||
        strequal(data->auth_host, data->hostname)) {
       sprintf(data->buffer, "%s:%s", data->user, data->passwd);
-      base64Encode(data->buffer, authorization);
-      data->ptr_userpwd = maprintf( "Authorization: Basic %s\015\012",
-                                    authorization);
+      if(base64_encode(data->buffer, strlen(data->buffer),
+                      &authorization) >= 0) {
+        data->ptr_userpwd = maprintf( "Authorization: Basic %s\015\012",
+                                      authorization);
+        free(authorization);
+      }
     }
   }
   if((data->bits.set_range) && !checkheaders(data, "Range:")) {
@@ -502,9 +523,9 @@ CURLcode http(struct connectdata *conn)
 
         /* and here comes the actual data */
         if(data->postfieldsize) {
-          ssend(data->firstsocket, data, "\r\n", 2);
-          ssend(data->firstsocket, data, data->postfields, data->postfieldsize);
-          ssend(data->firstsocket, data, "\r\n", 2);
+          ssend(data->firstsocket, conn, "\r\n", 2);
+          ssend(data->firstsocket, conn, data->postfields, data->postfieldsize);
+          ssend(data->firstsocket, conn, "\r\n", 2);
         }
         sendf(data->firstsocket, data,
               "\r\n"
