@@ -26,11 +26,11 @@
  * Contributor(s):
  *  Bjrn Reese <breese@mail1.stofanet.dk>
  *
- * 	http://curl.haxx.nu
+ * 	http://curl.haxx.se
  *
  * $Source: /cvsroot/curl/lib/ldap.c,v $
- * $Revision: 1.2 $
- * $Date: 2000/01/10 23:36:14 $
+ * $Revision: 1.8 $
+ * $Date: 2000/08/24 14:26:33 $
  * $Author: bagder $
  * $State: Exp $
  * $Locker:  $
@@ -38,6 +38,8 @@
  * ------------------------------------------------------------
  ****************************************************************************/
 
+#include "setup.h"
+
 /* -- WIN32 approved -- */
 #include <stdio.h>
 #include <string.h>
@@ -48,8 +50,6 @@
 #include <sys/stat.h>
 #include <errno.h>
 
-#include "setup.h"
-
 #if defined(WIN32) && !defined(__GNUC__)
 #else
 # ifdef HAVE_UNISTD_H
@@ -72,7 +72,7 @@
 #define DYNA_GET_FUNCTION(type, fnc) \
   (fnc) = (type)DynaGetFunction(#fnc); \
   if ((fnc) == NULL) { \
-    return URG_FUNCTION_NOT_FOUND; \
+    return CURLE_FUNCTION_NOT_FOUND; \
   } \
 
 /***********************************************************************
@@ -89,7 +89,18 @@ static void DynaOpen(void)
      * liblber.so automatically, but since it does not we will
      * handle it here by opening liblber.so as global.
      */
-    dlopen("liblber.so", RTLD_LAZY | RTLD_GLOBAL);
+    dlopen("liblber.so",
+#ifdef RTLD_LAZY_GLOBAL /* It turns out some systems use this: */
+           RTLD_LAZY_GLOBAL
+#else
+#ifdef RTLD_GLOBAL
+           RTLD_LAZY | RTLD_GLOBAL
+#else
+           /* and some systems don't have the RTLD_GLOBAL symbol */
+           RTLD_LAZY
+#endif
+#endif
+           );
     libldap = dlopen("libldap.so", RTLD_LAZY);
   }
 #endif
@@ -123,16 +134,21 @@ static void * DynaGetFunction(char *name)
 static int WriteProc(void *param, char *text, int len)
 {
   struct UrlData *data = (struct UrlData *)param;
-  
-  printf("%s\n", text);
+
+  data->fwrite(text, 1, strlen(text), data->out);
   return 0;
 }
 
+CURLcode ldap_done(struct connectdata *conn)
+{
+  return CURLE_OK;
+}
+
 /***********************************************************************
  */
-UrgError ldap(struct UrlData *data, char *path, long *bytecount)
+CURLcode ldap(struct connectdata *conn)
 {
-  UrgError status = URG_OK;
+  CURLcode status = CURLE_OK;
   int rc;
   void *(*ldap_open)(char *, int);
   int (*ldap_simple_bind_s)(void *, char *, char *);
@@ -146,24 +162,19 @@ UrgError ldap(struct UrlData *data, char *path, long *bytecount)
   void *server;
   void *result;
   void *entryIterator;
-#if 0
-  char *dn;
-  char **attrArray;
-  char *attrIterator;
-  char *attrString;
-  void *dummy;
-#endif
+
   int ldaptext;
+  struct UrlData *data=conn->data;
   
   infof(data, "LDAP: %s %s\n", data->url);
 
   DynaOpen();
   if (libldap == NULL) {
     failf(data, "The needed LDAP library/libraries couldn't be opened");
-    return URG_LIBRARY_NOT_FOUND;
+    return CURLE_LIBRARY_NOT_FOUND;
   }
 
-  ldaptext = data->conf & CONF_FTPASCII; /* This is a dirty hack */
+  ldaptext = data->bits.ftp_ascii; /* This is a dirty hack */
   
   /* The types are needed because ANSI C distinguishes between
    * pointer-to-object (data) and pointer-to-function.
@@ -182,17 +193,17 @@ UrgError ldap(struct UrlData *data, char *path, long *bytecount)
   if (server == NULL) {
     failf(data, "LDAP: Cannot connect to %s:%d",
 	  data->hostname, data->port);
-    status = URG_COULDNT_CONNECT;
+    status = CURLE_COULDNT_CONNECT;
   } else {
     rc = ldap_simple_bind_s(server, data->user, data->passwd);
     if (rc != 0) {
       failf(data, "LDAP: %s", ldap_err2string(rc));
-      status = URG_LDAP_CANNOT_BIND;
+      status = CURLE_LDAP_CANNOT_BIND;
     } else {
       rc = ldap_url_search_s(server, data->url, 0, &result);
       if (rc != 0) {
 	failf(data, "LDAP: %s", ldap_err2string(rc));
-	status = URG_LDAP_SEARCH_FAILED;
+	status = CURLE_LDAP_SEARCH_FAILED;
       } else {
 	for (entryIterator = ldap_first_entry(server, result);
 	     entryIterator;
@@ -204,7 +215,7 @@ UrgError ldap(struct UrlData *data, char *path, long *bytecount)
 				   "", 0, 0);
 	      if (rc != 0) {
 		failf(data, "LDAP: %s", ldap_err2string(rc));
-		status = URG_LDAP_SEARCH_FAILED;
+		status = CURLE_LDAP_SEARCH_FAILED;
 	      }
 	    } else {
 	      rc = ldap_entry2html(server, NULL, entryIterator, NULL,
@@ -212,7 +223,7 @@ UrgError ldap(struct UrlData *data, char *path, long *bytecount)
 				   "", 0, 0, NULL, NULL);
 	      if (rc != 0) {
 		failf(data, "LDAP: %s", ldap_err2string(rc));
-		status = URG_LDAP_SEARCH_FAILED;
+		status = CURLE_LDAP_SEARCH_FAILED;
 	      }
 	    }
 	  }
