@@ -31,8 +31,8 @@
  * 	http://curl.haxx.se
  *
  * $Source: /cvsroot/curl/lib/sendf.h,v $
- * $Revision: 1.4 $
- * $Date: 2000/08/24 12:33:17 $
+ * $Revision: 1.5 $
+ * $Date: 2000/09/21 08:49:16 $
  * $Author: bagder $
  * $State: Exp $
  * $Locker:  $
@@ -40,8 +40,9 @@
  * ------------------------------------------------------------
  ****************************************************************************/
 
+size_t ftpsendf(int fd, struct connectdata *, char *fmt, ...);
 size_t sendf(int fd, struct UrlData *, char *fmt, ...);
-size_t ssend(int fd, struct UrlData *, void *fmt, size_t len);
+size_t ssend(int fd, struct connectdata *, void *fmt, size_t len);
 void infof(struct UrlData *, char *fmt, ...);
 void failf(struct UrlData *, char *fmt, ...);
 
