@@ -24,13 +24,13 @@
  *
  * ------------------------------------------------------------
  * Main author:
- * - Daniel Stenberg <Daniel.Stenberg@haxx.nu>
+ * - Daniel Stenberg <daniel@haxx.se>
  *
- * 	http://curl.haxx.nu
+ * 	http://curl.haxx.se
  *
  * $Source: /cvsroot/curl/lib/telnet.c,v $
- * $Revision: 1.2 $
- * $Date: 2000/01/10 23:36:15 $
+ * $Revision: 1.5 $
+ * $Date: 2000/08/24 14:26:33 $
  * $Author: bagder $
  * $State: Exp $
  * $Locker:  $
@@ -38,7 +38,7 @@
  * ------------------------------------------------------------
  *
  * This implementation of the TELNET protocol is written by
- * Linus Nielsen <Linus.Nielsen@haxx.nu>,
+ * Linus Nielsen <Linus.Nielsen@haxx.se>,
  * with some code snippets stolen from the BSD Telnet client.
  *
  * The negotiation is performed according to RFC 1143 (D. Bernstein,
@@ -46,6 +46,8 @@
  *
  ****************************************************************************/
 
+#include "setup.h"
+
 /* -- WIN32 approved -- */
 #include <stdio.h>
 #include <string.h>
@@ -57,8 +59,6 @@
 
 #include <errno.h>
 
-#include "setup.h"
-
 #if defined(WIN32) && !defined(__GNUC__) || defined(__MINGW32__)
 #include <winsock.h>
 #include <time.h>
@@ -225,7 +225,7 @@ static void printoption(struct UrlData *data,
    char *fmt;
    char *opt;
    
-   if (data->conf & CONF_VERBOSE)
+   if (data->bits.verbose)
    {
       if (cmd == IAC)
       {
@@ -628,7 +628,7 @@ static void printsub(struct UrlData *data,
 {
    int i = 0;
 
-   if (data->conf & CONF_VERBOSE)
+   if (data->bits.verbose)
    {
       if (direction)
       {
@@ -871,23 +871,29 @@ void telwrite(struct UrlData *data,
    }
 }
 
-UrgError telnet(struct UrlData *data)
+CURLcode telnet_done(struct connectdata *conn)
 {
-   int sockfd = data->firstsocket;
-   fd_set readfd;
-   fd_set keepfd;
+  return CURLE_OK;
+}
 
-   bool keepon = TRUE;
-   char *buf = data->buffer;
-   int nread;
+CURLcode telnet(struct connectdata *conn)
+{
+  struct UrlData *data = conn->data;
+  int sockfd = data->firstsocket;
+  fd_set readfd;
+  fd_set keepfd;
 
-   init_telnet(data);
+  bool keepon = TRUE;
+  char *buf = data->buffer;
+  int nread;
+
+  init_telnet(data);
    
-   FD_ZERO (&readfd);		/* clear it */
-   FD_SET (sockfd, &readfd);
-   FD_SET (1, &readfd);
+  FD_ZERO (&readfd);		/* clear it */
+  FD_SET (sockfd, &readfd);
+  FD_SET (1, &readfd);
 
-   keepfd = readfd;
+  keepfd = readfd;
 
    while (keepon)
    {
@@ -931,7 +937,7 @@ UrgError telnet(struct UrlData *data)
 	 telrcv(data, (unsigned char *)buf, nread);
       }
    }
-   return URG_OK;
+   return CURLE_OK;
 }
 
 
