@@ -31,8 +31,8 @@
  * 	http://curl.haxx.se
  *
  * $Source: /cvsroot/curl/lib/urldata.h,v $
- * $Revision: 1.18 $
- * $Date: 2000/08/24 12:33:17 $
+ * $Revision: 1.22 $
+ * $Date: 2000/09/25 21:49:37 $
  * $Author: bagder $
  * $State: Exp $
  * $Locker:  $
@@ -123,6 +123,20 @@ typedef enum {
   CONN_LAST   /* illegal state */
 } ConnState;
 
+#ifdef KRB4
+struct krb4buffer {
+  void *data;
+  size_t size;
+  size_t index;
+  int eof_flag;
+};
+enum protection_level { 
+    prot_clear, 
+    prot_safe, 
+    prot_confidential, 
+    prot_private 
+};
+#endif
 
 /*
  * The connectdata struct contains all fields and variables that should be
@@ -150,6 +164,7 @@ struct connectdata {
 #define PROT_LDAP    (1<<7)
 #define PROT_FILE    (1<<8)
 
+  char *hostent_buf; /* pointer to allocated memory for name info */
   struct hostent *hp;
   struct sockaddr_in serv_addr;
   char proto[64];
@@ -188,6 +203,20 @@ struct connectdata {
                             the same we read from. -1 disables */
   long *writebytecountp; /* return number of bytes written or NULL */
 
+
+#ifdef KRB4
+
+  enum protection_level command_prot;
+  enum protection_level data_prot;
+  enum protection_level request_data_prot;
+
+  size_t buffer_size;
+
+  struct krb4buffer in_buffer, out_buffer;
+  int sec_complete;
+  void *app_data;
+
+#endif
 };
 
 struct Progress {
@@ -213,6 +242,11 @@ struct Progress {
   struct timeval t_connect;
   struct timeval t_pretransfer;
   int httpcode;
+
+#define CURR_TIME 5
+
+  double speeder[ CURR_TIME ];
+  int speeder_c;
 };
 
 /****************************************************************************
@@ -245,6 +279,7 @@ struct FTP {
 };
 
 struct Configbits {
+  bool tunnel_thru_httpproxy;
   bool ftp_append;
   bool ftp_ascii;
   bool ftp_list_only;
@@ -270,6 +305,7 @@ struct Configbits {
   bool user_passwd;
   bool verbose;
   bool this_is_a_follow; /* this is a followed Location: request */
+  bool krb4; /* kerberos4 connection requested */
 };
 
 /* What type of interface that intiated this struct */
@@ -355,6 +391,7 @@ struct UrlData {
   char *useragent;   /* User-Agent string */
 
   char *ftpport; /* port to send with the PORT command */
+  char *device;  /* Interface to use */
 
   /* function that stores the output:*/
   curl_write_callback fwrite;
@@ -444,6 +481,11 @@ struct UrlData {
   char *ptr_ref; /* free later if not NULL! */
   char *ptr_cookie; /* free later if not NULL! */
   char *ptr_host; /* free later if not NULL */
+
+  char *krb4_level;
+#ifdef KRB4
+  FILE *cmdchannel;
+#endif
 };
 
 #define LIBCURL_NAME "libcurl"
