@@ -29,8 +29,8 @@
  * 	http://curl.haxx.nu
  *
  * $Source: /cvsroot/curl/src/main.c,v $
- * $Revision: 1.29 $
- * $Date: 2000/08/24 17:56:20 $
+ * $Revision: 1.33 $
+ * $Date: 2000/09/25 22:20:02 $
  * $Author: bagder $
  * $State: Exp $
  * $Locker:  $
@@ -243,6 +243,8 @@ static void help(void)
        " -H/--header <line> Custom header to pass to server. (H)\n"
        " -i/--include       Include the HTTP-header in the output (H)\n"
        " -I/--head          Fetch document info only (HTTP HEAD/FTP SIZE)\n"
+       "    --interface <interface> Specify the interface to be used\n"
+       "    --krb4 <level>  Enable krb4 with specified security level (F)\n"
        " -K/--config        Specify which config file to read\n"
        " -l/--list-only     List only names of an FTP directory (F)\n"
        " -L/--location      Follow Location: hints (H)\n"
@@ -252,9 +254,7 @@ static void help(void)
        " -N/--no-buffer     Disables the buffering of the output stream\n"
        " -o/--output <file> Write output to <file> instead of stdout\n"
        " -O/--remote-name   Write output to a file named as the remote file\n"
-#if 0
-       " -p/--port <port>   Use port other than default for current protocol.\n"
-#endif
+       " -p/--proxytunnel   Perform non-HTTP services through a HTTP proxy\n"
        " -P/--ftpport <address> Use PORT with address instead of PASV when ftping (F)\n"
        " -q                 When used as the first parameter disables .curlrc\n"
        " -Q/--quote <cmd>   Send QUOTE command to FTP before file transfer (F)\n"
@@ -301,6 +301,7 @@ struct Configurable {
   char *headerfile;
   char remotefile;
   char *ftpport;
+  char *iface;
   unsigned short porttouse;
   char *range;
   int low_speed_limit;
@@ -311,6 +312,7 @@ struct Configurable {
   char *proxyuserpwd;
   char *proxy;
   bool configread;
+  bool proxytunnel;
   long conf;
   char *url;
   char *cert;
@@ -318,6 +320,7 @@ struct Configurable {
   bool crlf;
   char *cookiefile;
   char *customrequest;
+  char *krb4level;
   bool progressmode;
   bool nobuffer;
 
@@ -441,6 +444,8 @@ static int getparameter(char *flag, /* f or -long-flag */
   struct LongShort aliases[]= {
     {"9", "crlf",        FALSE},
     {"8", "stderr",      TRUE},
+    {"7", "interface",   TRUE},
+    {"6", "krb4",        TRUE},
 
     {"2", "sslv2",       FALSE},
     {"3", "sslv3",       FALSE},
@@ -473,9 +478,7 @@ static int getparameter(char *flag, /* f or -long-flag */
     {"N", "no-buffer",   FALSE},
     {"o", "output",      TRUE},
     {"O", "remote-name", FALSE},
-#if 0
-    {"p", "port",        TRUE},
-#endif
+    {"p", "proxytunnel", FALSE},
     {"P", "ftpport",     TRUE},
     {"q", "disable",     FALSE},
     {"Q", "quote",       TRUE},
@@ -612,6 +615,14 @@ static int getparameter(char *flag, /* f or -long-flag */
       else
         config->errors = stdout;
       break;
+    case '7': /* there is no short letter for this */
+      /* interface */
+      GetStr(&config->iface, nextarg);
+      break;
+    case '6': /* there is no short letter for this */
+      /* krb4 level string */
+      GetStr(&config->krb4level, nextarg);
+      break;
     case '#': /* added 19990617 larsa */
       config->progressmode ^= CURL_PROGRESS_BAR;
       break;
@@ -650,10 +661,12 @@ static int getparameter(char *flag, /* f or -long-flag */
     case 'c':
       /* This makes us continue an ftp transfer */
       config->use_resume^=TRUE;
+      fprintf(stderr, "-c is a deprecated switch, use '-C -' instead!\n");
       break;
     case 'C':
       /* This makes us continue an ftp transfer at given position */
-      config->resume_from= atoi(nextarg);
+      if(!strequal(nextarg, "-"))
+        config->resume_from= atoi(nextarg);
       config->use_resume=TRUE;
       break;
     case 'd':
@@ -789,15 +802,11 @@ static int getparameter(char *flag, /* f or -long-flag */
 	 */
       GetStr(&config->ftpport, nextarg);
       break;
-#if 0
     case 'p':
-      /* specified port */
-      fputs("You've used the -p option, it will be removed in a future version\n",
-	    stderr);
-      config->porttouse = atoi(nextarg);
-      config->conf |= CONF_PORT; /* changed port */
+      /* proxy tunnel for non-http protocols */
+      config->proxytunnel ^= TRUE;
       break;
-#endif
+
     case 'q': /* if used first, already taken care of, we do it like
 		 this so we don't cause an error! */
       break;
@@ -828,11 +837,14 @@ static int getparameter(char *flag, /* f or -long-flag */
     case 't':
       /* we are uploading */
       config->conf ^= CONF_UPLOAD;
+      fprintf(stderr, "-t is a deprecated switch, use '-T -' instead!\n");
       break;
     case 'T':
       /* we are uploading */
       config->conf |= CONF_UPLOAD;
-      GetStr(&config->infile, nextarg);
+      if(!strequal("-", nextarg))
+        /* make - equal stdin */
+        GetStr(&config->infile, nextarg);
       break;
     case 'u':
       /* user:password  */
@@ -1502,6 +1514,11 @@ int main(int argc, char *argv[])
     curl_easy_setopt(curl, CURLOPT_STDERR, config.errors);
     curl_easy_setopt(curl, CURLOPT_WRITEINFO, config.writeout);
 
+    /* three new ones in libcurl 7.3: */
+    curl_easy_setopt(curl, CURLOPT_HTTPPROXYTUNNEL, config.proxytunnel);
+    curl_easy_setopt(curl, CURLOPT_INTERFACE, config.iface);
+    curl_easy_setopt(curl, CURLOPT_KRB4LEVEL, config.krb4level);
+
     if((config.progressmode == CURL_PROGRESS_BAR) &&
        !(config.conf&(CONF_NOPROGRESS|CONF_MUTE))) {
       /* we want the alternative style, then we have to implement it
